using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
#pragma warning disable 1591	//stops XML warning


namespace QTPlugin
{
	/// <summary>
	/// Plugin type.
	/// </summary>
	public enum PluginType
	{
		/// <summary>
		/// Indicates that the plugin has toolbar item and instantialized only when the toolbar item is enabled.
		/// needs to inherit IBarButton or IBarCustomItem.
		/// </summary>
		Interactive,

		/// <summary>
		/// Indicates that the plugin is instantialized even if toolbar item is not enabled.
		/// This type plugin can have no toolbar item.
		/// If inherits IBarButton or IBarCustomItem, the plugin can have toolbar item.
		/// </summary>
		Background,

		/// <summary>
		/// Currently not used.
		/// </summary>
		Static, 
	}


	/// <summary>
	/// Code given when a plugin is about to be closed.
	/// Used in IPluginClient.Close()
	/// </summary>
	public enum EndCode
	{
		WindowClosed,		// Explorer window is closing.

		Unloaded,			// User disabled the plugin in Option window.
							// User removed the button from toolbar if the plugin type is PluginType.Interactive.

		Removed,			// User removed the plugin assembly in Option window of QTTabBar while plugins are instantialized.
							
		Hidden,				// If the plugin type is PluginType.Background and inherits IBarButton or IBarCustomItem, user removed button from toolbar.
							// Note that the plugin instance is still alive after 'Close' method called.
	}


	/// <summary>
	/// The type of menu to insert.
	/// </summary>
	[Flags]
	public enum MenuType
	{
		None = 0,
		Tab = 1,
		Bar = 2,
		Both = 3,
	}


	/// <summary>
	/// Commands to QTTabBar.
	/// </summary>
	public enum Commands
	{
		//[command]						[object arg]
		GoBack,							// int count
		GoForward,						// int count
		GoUpOneLevel,
		RefreshBrowser,

		CloseCurrentTab,
		CloseLeft,
		CloseRight,
		CloseAllButCurrent,
		CloseAllButOne,					// an ITab interface
		CloseWindow,
		UndoClose,
		BrowseFolder,
		ToggleTopMost,
		FocusFileList,

		OpenTabBarOptionDialog,
		OpenButtonBarOptionDialog,

		IsFolderTreeVisible,
		IsButtonBarVisible,
		ShowFolderTree,					// bool fShow (XP only)
		ShowButtonBar,					// NOT IMPLEMENTED  bool, fShow

		MD5,							// string[], file paths
		ShowProperties,					// an Address structure

		SetModalState,					// bool fShowing ( recommended to set this true/false before/after showing modal dialogboxes in the explorer thread. )
	}

	/* Note:
	 * 
	 * Currently, "Commands.CloseWindow", "Commands.OpenTabBarOptionDialog" and "Commands.OpenButtonBarOptionDialog" may unload the plugin
	 * that sent this command before IPluginServer.ExecuteCommand() method returns.
	 * 
	 * 
	 * Some shortcut keys still work while modal dialogboxes are shown.
	 * To prevent this, call IPluginServer.ExecuteCommand( Commands.SetModalState, true ) before Form.ShowDialog or MessageBox.Show and 
	 * call IPluginServer.ExecuteCommand( Commands.SetModalState, false ) after Form.ShowDialog or MessageBox.Show method returns.
	 * 
	 */


	/// <summary>
	/// Toolbar item display style.
	/// </summary>
	public enum DisplayStyle
	{
		ShowTextLabel,
		SelectiveText,
		NoLabel,
	}


	/// <summary>
	/// 
	/// </summary>
	public enum ExplorerWindowActions
	{
		None,
		Maximized,
		Minimized,
		Restored,
	}


	public static class CONSTANTS
	{
		public const string REGISTRY_PLUGIN = @"Software\Quizo\QTTabBar\Plugins";
		public const string REGISTRY_PLUGINSETTINGS = @"Software\Quizo\QTTabBar\Plugins\Settings";
	}

}
