using System;

namespace QTPlugin
{
	/// <summary>
	/// Specifies the informations about plugin.
	/// QTTabBar use this in option window.
	/// </summary>
	[AttributeUsage( AttributeTargets.Class )]
	public sealed class PluginAttribute : Attribute
	{
		/// <summary>
		/// Can be a nickname.
		/// </summary>
		public string Author;

		/// <summary>
		/// Name of the plugin. 
		/// </summary>
		public string Name;

		/// <summary>
		/// Version string like "1.0.0.0"
		/// </summary>
		public string Version;

		/// <summary>
		/// </summary>
		public string Description;

		/// <summary>
		/// Interactive or Background.
		/// </summary>
		public PluginType PluginType;

		/// <summary>
		/// </summary>
		/// <param name="pluginType"></param>
		public PluginAttribute( PluginType pluginType )
		{
			this.PluginType = pluginType;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="pluginType"></param>
		/// <param name="name"></param>
		/// <param name="author"></param>
		/// <param name="version"></param>
		/// <param name="description"></param>
		public PluginAttribute( PluginType pluginType, string name, string author, string version, string description )
		{
			this.PluginType = pluginType;
			this.Name = name;
			this.Author = author;
			this.Version = version;
			this.Description = description;
		}
	}
}
